/*
 * Copyright (c) 2012 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global window, setTimeout, tizen, document, Image, console */

/**
 * Piano object variable
 * @type Piano
 */
var piano;

/**
 * Piano class constructor.
 * @constructor
 */
function Piano() {
    'use strict';
    return;
}

// Definition of the Piano class
(function strict() {
    'use strict';

    Piano.prototype = {
        /**
         * Maps multitouch fingers to piano keys.
         * @type Array
         */
        touchPianoKey: [],
        /**
         * Audio tags for keys.
         * @type Array
         */
        audio: [],
        /**
         * Cache for pressed white piano key image.
         * @type HTMLImageElement
         */
        whiteCache: null,
        /**
         * Cache for pressed black piano key image.
         * @type HTMLImageElement
         */
        blackCache: null
    };

    /**
     * Binds UI events to handler functions.
     */
    Piano.prototype.bindEvents = function bindEvents() {
        var keys = document.getElementsByTagName('body')[0]; // keys container

        // Bind piano key press.
        keys.addEventListener(
            'touchstart',
            this.onPianoKeyTouchDown.bind(this)
        );
        keys.addEventListener(
            'touchmove',
            this.onPianoKeyTouchDown.bind(this)
        );

        // Bind piano key release.
        keys.addEventListener(
            'touchend',
            this.onPianoKeyTouchUp.bind(this)
        );

        // Bind hardware back button.
        window.addEventListener('tizenhwkey', function onTizenHwKey(e) {
            if (e.keyName === 'back') {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (err) {
                    console.log('Error: ', err);
                }
            }
        });
    };

    /**
     * Handles piano key press.
     * @param {Event} eventData Event data for the press touch event
     */
    Piano.prototype.onPianoKeyTouchDown =
        function onPianoKeyTouchDown(eventData) {
            eventData.preventDefault();
            var touch,
                element,
                len = eventData.changedTouches.length,
                i,
                lastKeyId;
            for (i = 0; i < len; i += 1) {
                // Find multitouch finger touch event
                touch = eventData.changedTouches[i];
                // Find piano key under this finger
                element = document.elementFromPoint(
                    touch.clientX,
                    touch.clientY
                );

                if (element &&
                        this.touchPianoKey[touch.identifier] !== element.id) {
                    // Add/modify mapping for multitouch finger identifier
                    // to the piano key
                    lastKeyId = this.touchPianoKey[touch.identifier];
                    this.touchPianoKey[touch.identifier] = element.id;
                    // Release the last key under this finger
                    setTimeout(this.releaseKey.bind(this, lastKeyId), 10);
                    // Play audio file for this piano key
                    this.playAudio(element.id.substr(3));
                    // Change key's state to pressed
                    element.classList.add('pressed');
                }
            }
        };

    /**
     * Handles piano key release.
     * @param {Event} eventData Event data for the release touch event
     */
    Piano.prototype.onPianoKeyTouchUp = function onPianoKeyTouchUp(eventData) {
        var touchId,
            keyId,
            len = eventData.changedTouches.length,
            i;
        for (i = 0; i < len; i += 1) {
            // Find multitouch finger touch event identifier
            touchId = eventData.changedTouches[i].identifier;
            // Find piano key that was pressed
            keyId = this.touchPianoKey[touchId];
            // Remove mapping for multitouch finger identifier to the piano key
            this.touchPianoKey[touchId] = undefined;
            // Release the piano key
            setTimeout(this.releaseKey.bind(this, keyId), 10);
        }
    };

    /**
     * Releases given piano key if it is no longer pressed.
     * @param {string} keyId Piano key's element id
     */
    Piano.prototype.releaseKey = function releaseKey(keyId) {
        // Change key's state to normal
        var stillPressed = false,
            len = this.touchPianoKey.length,
            i;

        if (keyId === undefined) {
            return;
        }

        // Check for every multitouch event fingers
        for (i = 0; i < len; i += 1) {
            if (this.touchPianoKey[i] === keyId) {
                // This piano key is still pressed with another finger
                stillPressed = true;
                break;
            }
        }
        // If key is no longer pressed, change its state to normal
        if (!stillPressed) {
            document.getElementById(keyId).classList.remove('pressed');
        }
    };

    /**
     * Initializes audio tags.
     */
    Piano.prototype.audioInit = function audioInit() {
        var i;
        for (i = 0; i <= 13; i += 1) {
            this.audio[i] = document.createElement('audio');
            this.audio[i].src = 'sounds/' + i + '.wav';
        }
    };

    /**
     * Plays sound for specified audio.
     * @param {number} audioId Number of the audio tag
     */
    Piano.prototype.playAudio = function playAudio(audioId) {
        var audio = this.audio[audioId];
        if (audio) {
            if (audio.currentTime !== 0) {
                audio.currentTime = 0;
            }
            if (audio.paused) {
                audio.play();
            }
        }
    };

    /**
     * Preloads pressed piano keys images.
     */
    Piano.prototype.cacheImages = function cacheImages() {
        this.whiteCache = new Image();
        this.whiteCache.src = 'images/white_pressed.png';
        this.blackCache = new Image();
        this.blackCache.src = 'images/black_pressed.png';
    };

    /**
     * Initializes the Piano application.
     */
    Piano.prototype.init = function init() {
        this.cacheImages();
        this.bindEvents();
        this.audioInit();
    };

}());

// Create Piano class instance
piano = new Piano();

// Initialize Piano when the document is ready
document.addEventListener('DOMContentLoaded', function onDocumentReady() {
    'use strict';
    piano.init();
});
